from pathlib import Path
from unittest import TestCase

import remotior_sensus


class TestRasterEdit(TestCase):

    def test_raster_edit(self):
        rs = remotior_sensus.Session(
            n_processes=2, available_ram=1000, log_level=10
            )
        cfg = rs.configurations
        data_path = Path(__file__).parent / 'data'
        p = str(data_path / 'S2_2020-01-01' / 'S2_B02.tif')
        v = str(data_path / 'files' / 'roi.gpkg')
        cfg.logger.log.debug('>>> test raster edit')
        temp = cfg.temp.temporary_file_path(name_suffix=cfg.tif_suffix)
        rs.files_directories.copy_file(in_path=p, out_path=temp)
        edit = rs.raster_edit(
            raster_path=temp, vector_path=v, constant_value=10
        )
        self.assertTrue(edit.extra['column_start'] > 0)
        cfg.logger.log.debug('>>> test undo raster edit')
        edit_2 = rs.raster_edit(
            raster_path=temp, column_start=edit.extra['column_start'],
            row_start=edit.extra['row_start'],
            old_array=edit.extra['old_array']
        )
        self.assertTrue(edit_2.extra['column_start'] is None)
        cfg.logger.log.debug('>>> test raster edit with expression')
        temp_3 = cfg.temp.temporary_file_path(name_suffix=cfg.tif_suffix)
        rs.files_directories.copy_file(in_path=p, out_path=temp_3)
        expression = 'where(%s%s%s > 500, 10, %s%s%s)' % (
            cfg.variable_band_quotes, cfg.variable_raster_name,
            cfg.variable_band_quotes, cfg.variable_band_quotes,
            cfg.variable_raster_name, cfg.variable_band_quotes
        )
        edit_3 = rs.raster_edit(
            raster_path=temp_3, vector_path=v, expression=expression
        )
        self.assertTrue(edit_3.extra['column_start'] > 0)

        # clear temporary directory
        rs.close()



from unittest import TestCase
import numpy as np
from pathlib import Path
import multiprocessing
from remotior_sensus.core import processor
import remotior_sensus
# noinspection PyProtectedMember
from remotior_sensus.core.multiprocess_manager import (
    _calculate_block_size, _compute_raster_pieces
)
from remotior_sensus.core.processor_functions import (
    reclassify_raster, band_calculation
)


class TestProcessor(TestCase):

    def test_processor(self):
        rs = remotior_sensus.Session(
            n_processes=2, available_ram=1000, log_level=10
            )
        cfg = rs.configurations
        cfg.logger.log.debug('>>> test reclassification')
        process_parameters = [
            1, cfg.temp, 500, cfg.gdal_path, multiprocessing.Manager().Queue(),
            cfg.refresh_time, cfg.memory_unit_array_12, cfg.log_level,
            None, None
        ]
        data_path = Path(__file__).parent / 'data'
        raster_path = str(data_path / 'S2_2020-01-01' / 'S2_B02.tif')
        calc_datatype = [np.float32]
        input_nodata_as_value = False
        use_value_as_nodata = [1]
        (raster_x_size, raster_y_size, block_size_x, block_size_y,
         list_range_x, list_range_y, tot_blocks,
         number_of_bands) = _calculate_block_size(
            raster_path, 1, cfg.memory_unit_array_12, 0,
            available_ram=100
        )
        # compute raster pieces
        pieces = _compute_raster_pieces(
            raster_x_size, raster_y_size, block_size_x, block_size_y,
            list_range_y, 1, separate_bands=False,
            unique_section=True
        )
        # use raster files directly (not vrt)
        input_parameters = [[[raster_path]], calc_datatype, None,
                            pieces[0], None, None,
                            use_value_as_nodata, None,
                            input_nodata_as_value, None,
                            0, None, True]
        temp = cfg.temp.temporary_file_path(name_suffix=cfg.tif_suffix)
        compress = cfg.raster_compression
        compress_format = cfg.raster_compression_format
        any_nodata_mask = False
        output_nodata_value = -10
        keep_output_array = True
        keep_output_argument = False
        output_parameters = [[temp], [cfg.raster_data_type],
                             compress, compress_format, any_nodata_mask,
                             [output_nodata_value], [1],
                             keep_output_array, keep_output_argument]
        dtype_list = [(cfg.old_value, 'U1024'), (cfg.new_value, 'U1024')]
        function = reclassify_raster
        function_argument = np.array(
            ([('1', '2'), ('1110', '3')]), dtype=dtype_list
        )
        function_variable = cfg.variable_raster_name
        (output_array_list, out_files, proc_error,
         logger) = processor.function_initiator(
            process_parameters, input_parameters, output_parameters,
            function, [function_argument], [function_variable],
            False, False, False,
            False,
        )
        self.assertEqual(output_array_list[0][0][0, 0], output_nodata_value)
        self.assertTrue(rs.files_directories.is_file(out_files[0][0]))
        cfg.logger.log.debug('>>> test band calc')
        function = band_calculation
        function_argument = '_array_function_placeholder * 2'
        function_variable = cfg.variable_raster_name
        (output_array_list, out_files, proc_error,
         logger) = processor.function_initiator(
            process_parameters, input_parameters, output_parameters,
            function, [function_argument], [function_variable],
            False, False, False,
            False
        )
        self.assertEqual(output_array_list[0][0][0, 0], output_nodata_value)
        self.assertTrue(rs.files_directories.is_file(out_files[0][0]))

        # clear temporary directory
        rs.close()



# Remotior Sensus , software to process remote sensing and GIS data.
# Copyright (C) 2022-2024 Luca Congedo.
# Author: Luca Congedo
# Email: ing.congedoluca@gmail.com
#
# This file is part of Remotior Sensus.
# Remotior Sensus is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
# Remotior Sensus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Remotior Sensus. If not, see <https://www.gnu.org/licenses/>.
"""
Raster edit.

This tool allows for the direct editing of a raster band.
Warning, this tool edits the original input raster. 

Typical usage example:

    >>> # import Remotior Sensus and start the session
    >>> import remotior_sensus
    >>> rs = remotior_sensus.Session()
    >>> # start the process
    >>> edit = rs.raster_edit(raster_path='input_path', 
    ... vector_path='vector_path')
"""  # noqa: E501

from typing import Union, Optional

import numpy as np

from remotior_sensus.core import configurations as cfg
from remotior_sensus.core.output_manager import OutputManager
from remotior_sensus.util import (
    files_directories, raster_vector, shared_tools
)
from remotior_sensus.core.processor_functions import edit_raster


def raster_edit(
        raster_path: str, vector_path: Optional[str] = None,
        field_name: Optional[str] = None,
        expression: Optional[str] = None,
        constant_value: Optional[Union[int, float]] = None,
        old_array: Optional[np.ndarray] = None,
        column_start: Optional[int] = None, row_start: Optional[int] = None,
        available_ram: Optional[int] = None,
        progress_message: Optional[bool] = True
) -> OutputManager:
    """Edit a raster band based on a vector file.

    This tool allows for editing the values of a raster using a vector file.
    All pixel intersecting the vector will be edited with a new value.
    New pixel values can be a constant value or defined by a custom expression defining a condition.
    The input raster is directly edited, without producing a new file.

    Args:
        raster_path: path of raster used as input.
        vector_path: string of vector path (optional for undo operation).
        field_name: string of vector field name.
        expression: optional conditional expression; pixels are edited if 
            intersecting the vector and the expression is True; the variable 
            "raster" is used to refer to the raster pixel values;
            e.g. where("raster" > 500, 10, "raster").
        constant_value: optional new value for pixels.
        old_array: optional previous array for undo operation.
        column_start: optional starting column for undo operation.
        row_start: optional starting row for undo operation.
        available_ram: number of megabytes of RAM available to processes.
        progress_message: if True then start progress message, if False does 
            not start the progress message (useful if launched from other tools).

    Returns:
        object :func:`~remotior_sensus.core.output_manager.OutputManager` with
            - extra = {'old_array': previous array for undo operation, 
              'column_start': starting column for undo operation, 
              'row_start': starting row for undo operation}

    Examples:
        Perform the edit of a raster
            >>> edit_1 = raster_edit(raster_path='input_path', 
            ... vector_path='output_path')
            
        Perform the undo of edit of a raster
            >>> edit_2 = raster_edit(raster_path='input_path', 
            ... column_start=edit_1.extra['column_start'],
            ... row_start=edit_1.extra['row_start'],
            ... old_array=edit_1.extra['old_array']
            ... )
            
        Perform the edit of a raster with condition
            >>> edit_3 = raster_edit(raster_path='input_path', 
            ... vector_path='output_path', 
            ... expression='where("raster" > 500, 10, "raster")'
            ... )
            
    """  # noqa: E501
    cfg.logger.log.info('start')
    cfg.progress.update(
        process=__name__.split('.')[-1].replace('_', ' '), message='starting',
        start=progress_message
    )
    raster_path = files_directories.input_path(raster_path)
    # prepare process files
    prepared = shared_tools.prepare_process_files(
        input_bands=[raster_path], output_path=None
    )
    raster_info = prepared['raster_info']
    input_raster_list = prepared['input_raster_list']
    if vector_path is not None:
        # get vector info
        vector, raster, mask_crs = raster_vector.raster_or_vector_input(
            vector_path
        )
        # check crs
        same_crs = raster_vector.compare_crs(raster_info[0][1], mask_crs)
        if not same_crs:
            # project vector to raster crs
            t_vector = cfg.temp.temporary_file_path(
                name_suffix=cfg.gpkg_suffix
            )
            try:
                raster_vector.reproject_vector(
                    vector_path, t_vector, raster_info[0][1], mask_crs
                )
            except Exception as err:
                cfg.logger.log.error(str(err))
                cfg.messages.error(str(err))
                cfg.progress.update(failed=True)
                return OutputManager(check=False)
            vector_path = t_vector
    # convert vector to raster
    vector_raster = cfg.temp.temporary_raster_path(
        extension=cfg.tif_suffix
    )
    if field_name is None:
        burn_values = 1
    else:
        burn_values = None
    # perform conversion
    cfg.multiprocess.multiprocess_vector_to_raster(
        vector_path=vector_path, output_path=vector_raster,
        field_name=field_name, burn_values=burn_values,
        nodata_value=0, background_value=0,
        reference_raster_path=input_raster_list[0], minimum_extent=True,
        available_ram=available_ram
    )
    if expression is not None:
        cfg.logger.log.debug('expression: %s' % expression)
        check, expression = check_expression(expression, constant_value)
        if check is False:
            cfg.logger.log.error('invalid expression')
            cfg.messages.error('invalid expression')
            cfg.progress.update(failed=True)
            return OutputManager(check=False)
        cfg.logger.log.debug('expression: %s' % expression)
    # run edit raster
    argument_list = [{
        'input_raster': input_raster_list[0], 'vector_raster': vector_raster,
        'constant_value': constant_value, 'expression': expression,
        'old_array': old_array, 'column_start': column_start,
        'row_start': row_start, 'gdal_path': cfg.gdal_path
    }]
    function_list = [edit_raster]
    cfg.multiprocess.run_iterative_process(
        function_list=function_list, argument_list=argument_list
    )
    output = cfg.multiprocess.output
    if output is not False:
        old_array, column_start, row_start = output[0]
    cfg.progress.update(end=True)
    cfg.logger.log.info('end; raster edit: %s' % str(raster_path))
    return OutputManager(extra={
        'old_array': old_array, 'column_start': column_start,
        'row_start': row_start,
        })


# check the expression
def check_expression(expression, constant_value):
    # expose numpy functions
    log = np.log
    _log = log
    log10 = np.log10
    _log10 = log10
    sqrt = np.sqrt
    _sqrt = sqrt
    cos = np.cos
    _cos = cos
    arccos = np.arccos
    _arccos = arccos
    sin = np.sin
    _sin = sin
    arcsin = np.arcsin
    _arcsin = arcsin
    tan = np.tan
    _tan = tan
    arctan = np.arctan
    _arctan = arctan
    exp = np.exp
    _exp = exp
    min = np.nanmin
    _min = min
    max = np.nanmax
    _max = max
    sum = np.nansum
    _sum = sum
    percentile = np.nanpercentile
    _percentile = percentile
    median = np.nanmedian
    _median = median
    mean = np.nanmean
    _mean = mean
    std = np.nanstd
    _std = std
    where = np.where
    _where = where
    nan = np.nan
    _nan = nan
    # replace variables
    expression = expression.replace(
        '%s%s%s' % (cfg.variable_band_quotes, cfg.variable_vector_name,
                    cfg.variable_band_quotes), str(constant_value)
    )
    expression = expression.replace(
        '%s%s%s' % (cfg.variable_band_quotes, cfg.variable_raster_name,
                    cfg.variable_band_quotes), cfg.variable_raster_name
    )
    conditional_exp = 'np.where(%s> 0 , %s, %s)' % (
        cfg.variable_vector_name, expression, cfg.variable_raster_name
    )
    # test array
    _test_array = np.arange(9).reshape(3, 3)
    expression_copy = conditional_exp
    expression_copy = expression_copy.replace(cfg.variable_vector_name,
                                              '_test_array')
    expression_copy = expression_copy.replace(cfg.variable_raster_name,
                                              '_test_array')
    try:
        eval(expression_copy)
        return True, conditional_exp
    except Exception as err:
        cfg.logger.log.error(str(err))
        return False, None



from pathlib import Path
from unittest import TestCase

import remotior_sensus


class TestBandSpectralDistance(TestCase):

    def test_band_spectral_distance(self):
        rs = remotior_sensus.Session(
            n_processes=2, available_ram=1000, log_level=10
            )
        cfg = rs.configurations
        catalog = rs.bandset_catalog()
        data_path = Path(__file__).parent / 'data'
        file_list_1 = ['S2_2020-01-03/S2_B02.tif', 'S2_2020-01-03/S2_B03.tif',
                       'S2_2020-01-03/S2_B04.tif']
        file_list_2 = ['S2_2020-01-04/S2_B02.tif', 'S2_2020-01-04/S2_B03.tif',
                       'S2_2020-01-04/S2_B04.tif']
        catalog.create_bandset(
            file_list_1, wavelengths=['Sentinel-2'], bandset_number=1,
            root_directory=str(data_path)
            )
        catalog.create_bandset(
            file_list_2, wavelengths=['Sentinel-2'], bandset_number=2,
            root_directory=str(data_path)
            )
        bandset_list = [catalog.get_bandset(1), catalog.get_bandset(2)]
        cfg.logger.log.debug('>>> test spectral distance')
        temp = cfg.temp.temporary_file_path(name_suffix=cfg.tif_suffix)
        distance = rs.band_spectral_distance(
            input_bandsets=bandset_list, output_path=temp
        )
        self.assertTrue(rs.files_directories.is_file(distance.path))
        self.assertTrue(distance.check)
        cfg.logger.log.debug('>>> test spectral distance with catalog')
        temp = cfg.temp.temporary_file_path(name_suffix=cfg.tif_suffix)
        distance = rs.band_spectral_distance(
            input_bandsets=[1, 2], output_path=temp, bandset_catalog=catalog
        )
        self.assertTrue(rs.files_directories.is_file(distance.path))
        self.assertTrue(distance.check)
        cfg.logger.log.debug('>>> test spectral distance using spectral angle')
        temp = cfg.temp.temporary_file_path(name_suffix=cfg.tif_suffix)
        distance = rs.band_spectral_distance(
            input_bandsets=bandset_list, output_path=temp,
            algorithm_name=cfg.spectral_angle_mapping_a
        )
        self.assertTrue(rs.files_directories.is_file(distance.path))
        self.assertTrue(distance.check)
        cfg.logger.log.debug('>>> test spectral distance with threshold')
        temp = cfg.temp.temporary_file_path(name_suffix=cfg.tif_suffix)
        distance = rs.band_spectral_distance(
            input_bandsets=bandset_list, output_path=temp, threshold=1000
        )
        self.assertTrue(rs.files_directories.is_file(distance.path))
        self.assertTrue(distance.check)

        # clear temporary directory
        rs.close()



# Remotior Sensus , software to process remote sensing and GIS data.
# Copyright (C) 2022-2024 Luca Congedo.
# Author: Luca Congedo
# Email: ing.congedoluca@gmail.com
#
# This file is part of Remotior Sensus.
# Remotior Sensus is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by 
# the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
# Remotior Sensus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty 
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Remotior Sensus. If not, see <https://www.gnu.org/licenses/>.
"""Band spectral distance.

This tool allows for calculating the spectral distance pixel by pixel between 
two BandSets.

Typical usage example:

    >>> # import Remotior Sensus and start the session
    >>> import remotior_sensus
    >>> rs = remotior_sensus.Session()
    >>> catalog = rs.bandset_catalog()
    >>> # create three BandSets
    >>> file_list_1 = ['file1_b1.tif', 'file1_b2.tif', 'file1_b3.tif']
    >>> file_list_2 = ['file2_b1.tif', 'file2_b2.tif', 'file2_b3.tif']
    >>> catalog.create_bandset(file_list_1, bandset_number=1)
    >>> catalog.create_bandset(file_list_2, bandset_number=2)
    >>> distance = rs.band_spectral_distance(
    ... input_bandsets=[catalog.get(1), catalog.get(2)], output_path='output_path',
    ... )
"""  # noqa: E501

from typing import Optional

from remotior_sensus.core import configurations as cfg
from remotior_sensus.core.bandset_catalog import BandSet
from remotior_sensus.core.bandset_catalog import BandSetCatalog
from remotior_sensus.core.output_manager import OutputManager
from remotior_sensus.core.processor_functions import spectral_distance
from remotior_sensus.util import shared_tools


def band_spectral_distance(
        input_bandsets: list, output_path: Optional[str] = None,
        algorithm_name: Optional[str] = None,
        nodata_value: Optional[int] = None, threshold: Optional[float] = None,
        n_processes: Optional[int] = None, available_ram: Optional[int] = None,
        virtual_output: Optional[bool] = False,
        overwrite: Optional[bool] = False,
        bandset_catalog: Optional[BandSetCatalog] = None,
        progress_message: Optional[bool] = True
) -> OutputManager:
    """Band spectral distance.

    This tool allows for calculating the spectral distance pixel by pixel 
    between two BandSets, which can be useful for change detection.
    A new raster is created where pixel values rapresent the spectral distance 
    between pixels of input BandSets.
    Optionally, a threshold value can be defined to create a binary raster 
    where pixel value is 1 if distance > threshold or 0 otherwise.

    Args:
        input_bandsets: list of number of BandSets, or BandSet objects.
        output_path: string of output path directory.
        algorithm_name: algorithm name selected form cfg.classification_algorithms.
        nodata_value: value to be considered as nodata.
        threshold: threshold value to create a binary raster if distance > threshold.
        n_processes: number of parallel processes.
        available_ram: number of megabytes of RAM available to processes.
        virtual_output: if True (and output_path is directory), save output
            as virtual raster of multiprocess parts.
        overwrite: if True, output overwrites existing files.
        bandset_catalog: optional type BandSetCatalog for BandSet number.
        progress_message: if True then start progress message, if False does 
            not start the progress message (useful if launched from other tools).

    Returns:
        Object :func:`~remotior_sensus.core.output_manager.OutputManager` with
            - path = output path

    Examples:
        Perform the spectral distance of two BandSets using threshold
            >>> # import Remotior Sensus and start the session
            >>> import remotior_sensus
            >>> rs = remotior_sensus.Session()
            >>> catalog = rs.bandset_catalog()
            >>> # create three BandSets
            >>> file_list_1 = ['file1_b1.tif', 'file1_b2.tif', 'file1_b3.tif']
            >>> file_list_2 = ['file2_b1.tif', 'file2_b2.tif', 'file2_b3.tif']
            >>> catalog.create_bandset(file_list_1, bandset_number=1)
            >>> catalog.create_bandset(file_list_2, bandset_number=2)
            >>> # start the process
            >>> distance = rs.band_spectral_distance(
            ... input_bandsets=[1, 2], output_path='output_path', 
            ... threshold=1000, bandset_catalog=catalog
            ... )
    """  # noqa: E501
    cfg.logger.log.info('start')
    cfg.progress.update(
        process=__name__.split('.')[-1].replace('_', ' '), message='starting',
        start=progress_message
    )
    cfg.logger.log.debug('input_bands: %s' % str(input_bandsets))
    if n_processes is None:
        n_processes = cfg.n_processes
    if algorithm_name is None:
        algorithm_name = cfg.minimum_distance_a
    elif algorithm_name.lower() == cfg.minimum_distance:
        algorithm_name = cfg.minimum_distance_a
    elif algorithm_name.lower() == cfg.minimum_distance_a:
        algorithm_name = cfg.minimum_distance_a
    elif algorithm_name.lower() == cfg.spectral_angle_mapping:
        algorithm_name = cfg.spectral_angle_mapping_a
    elif algorithm_name.lower() == cfg.spectral_angle_mapping_a:
        algorithm_name = cfg.spectral_angle_mapping_a
    else:
        cfg.logger.log.error('algorithm name')
        cfg.messages.error('algorithm name')
        cfg.progress.update(failed=True)
        return OutputManager(check=False)
    # list of band lists to mosaic
    if len(input_bandsets) != 2:
        cfg.logger.log.error('input bandsets')
        cfg.messages.error('input bandsets')
        cfg.progress.update(failed=True)
        return OutputManager(check=False)
    combination_band_list = []
    band_extended_list = []
    for i in input_bandsets:
        # list of band sets
        if type(i) is BandSet or type(i) is int:
            # get input list
            if i is None:
                cfg.logger.log.error('input None:%s' % str(i))
                cfg.messages.error('input None:%s' % str(i))
                cfg.progress.update(failed=True)
                return OutputManager(check=False)
            else:
                band_list = BandSetCatalog.get_band_list(i, bandset_catalog)
                combination_band_list.append(band_list)
                band_extended_list.extend(band_list)
        # list of raster paths
        else:
            cfg.logger.log.error('input bands')
            cfg.messages.error('input bands')
            cfg.progress.update(failed=True)
            return OutputManager(check=False)
    if len(combination_band_list) == 0:
        cfg.logger.log.error('input bands')
        cfg.messages.error('input bands')
        cfg.progress.update(failed=True)
        return OutputManager(check=False)
    if len(combination_band_list[0]) != len(combination_band_list[1]):
        cfg.logger.log.error('input bands')
        cfg.messages.error('input bands')
        cfg.progress.update(failed=True)
        return OutputManager(check=False)
    # prepare process files
    prepared = shared_tools.prepare_process_files(
        input_bands=band_extended_list, output_path=output_path,
        multiple_input=True, overwrite=overwrite, n_processes=n_processes,
        bandset_catalog=bandset_catalog
    )
    out_path = prepared['output_path']
    n_processes = prepared['n_processes']
    vrt_path = prepared['temporary_virtual_raster']
    # calculate spectral distance
    cfg.multiprocess.run(
        raster_path=vrt_path, function=spectral_distance,
        function_argument=len(combination_band_list[0]),
        output_raster_path=out_path,
        function_variable=[algorithm_name, threshold],
        use_value_as_nodata=nodata_value, any_nodata_mask=True,
        virtual_raster=virtual_output, n_processes=n_processes,
        available_ram=available_ram,
        progress_message='calculate spectral distance'
    )
    cfg.progress.update(end=True)
    cfg.logger.log.info('end; band spectral distance: %s' % str(out_path))
    return OutputManager(path=out_path)



from pathlib import Path
from unittest import TestCase

import remotior_sensus


class TestMosaicBands(TestCase):

    def test_mosaic_bands(self):
        rs = remotior_sensus.Session(
            n_processes=2, available_ram=1000, log_level=10
            )
        cfg = rs.configurations
        catalog = rs.bandset_catalog()
        data_path = Path(__file__).parent / 'data'
        file_list = [
            str(data_path / 'S2_2020-01-01' / 'S2_B02.tif'),
            str(data_path / 'S2_2020-01-02' / 'S2_B02.tif')
        ]
        temp = cfg.temp.dir
        cfg.logger.log.debug('>>> test mosaic')
        mosaic = rs.mosaic(file_list, temp)
        self.assertTrue(rs.files_directories.is_file(mosaic.paths[0]))
        self.assertTrue(mosaic.check)
        file_list_1 = ['S2_2020-01-01/S2_B02.tif', 'S2_2020-01-01/S2_B03.tif',
                       'S2_2020-01-01/S2_B04.tif']
        file_list_2 = ['S2_2020-01-02/S2_B02.tif', 'S2_2020-01-02/S2_B03.tif',
                       'S2_2020-01-02/S2_B04.tif']
        file_list_3 = ['S2_2020-01-03/S2_B02.tif', 'S2_2020-01-03/S2_B03.tif',
                       'S2_2020-01-03/S2_B04.tif']
        catalog.create_bandset(
            file_list_1, wavelengths=['Sentinel-2'], bandset_number=1,
            root_directory=str(data_path)
            )
        catalog.create_bandset(
            file_list_2, wavelengths=['Sentinel-2'], bandset_number=2,
            root_directory=str(data_path)
            )
        catalog.create_bandset(
            file_list_3, wavelengths=['Sentinel-2'], bandset_number=3,
            root_directory=str(data_path)
            )
        bandset_list = [catalog.get_bandset(1), catalog.get_bandset(2),
                        catalog.get_bandset(3)]
        cfg.logger.log.debug('>>> test mosaic BandSet')
        mosaic = rs.mosaic(bandset_list, temp)
        self.assertTrue(rs.files_directories.is_file(mosaic.paths[0]))
        self.assertTrue(mosaic.check)
        bandset_list = [1, 2]
        mosaic = rs.mosaic(
            bandset_list, output_path=temp, bandset_catalog=catalog
            )
        self.assertTrue(rs.files_directories.is_file(mosaic.paths[0]))
        self.assertTrue(mosaic.check)

        band_list_1 = [
            str(data_path / 'S2_2020-01-01' / 'S2_B02.tif'),
            str(data_path / 'S2_2020-01-02' / 'S2_B02.tif'),
            str(data_path / 'S2_2020-01-02' / 'S2_B02.tif')
        ]
        band_list_2 = [
            str(data_path / 'S2_2020-01-01' / 'S2_B03.tif'),
            str(data_path / 'S2_2020-01-02' / 'S2_B03.tif'),
            str(data_path / 'S2_2020-01-02' / 'S2_B03.tif')
        ]
        band_list_3 = [
            str(data_path / 'S2_2020-01-01' / 'S2_B04.tif'),
            str(data_path / 'S2_2020-01-02' / 'S2_B04.tif'),
            str(data_path / 'S2_2020-01-02' / 'S2_B04.tif')
        ]
        band_list = [band_list_1, band_list_2, band_list_3]
        mosaic = rs.mosaic(
            band_list, output_path=temp, bandset_catalog=catalog,
            prefix='prefix', output_name='output_name'
            )
        self.assertTrue(rs.files_directories.is_file(mosaic.paths[0]))
        self.assertTrue(mosaic.check)

        # clear temporary directory
        rs.close()



# Remotior Sensus , software to process remote sensing and GIS data.
# Copyright (C) 2022-2024 Luca Congedo.
# Author: Luca Congedo
# Email: ing.congedoluca@gmail.com
#
# This file is part of Remotior Sensus.
# Remotior Sensus is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by 
# the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
# Remotior Sensus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty 
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Remotior Sensus. If not, see <https://www.gnu.org/licenses/>.
"""Band erosion.

This tool allows for the spatial erosion, through a moving window,
of band pixels selected by values.

Typical usage example:

    >>> # import Remotior Sensus and start the session
    >>> import remotior_sensus
    >>> rs = remotior_sensus.Session()
    >>> # start the process
    >>> erosion = rs.band_erosion(input_bands=['file1.tif', 'file2.tif'],
    ... value_list=[1], size=1, output_path='directory_path',
    ... circular_structure=True,prefix='erosion_')
"""

from typing import Union, Optional

from remotior_sensus.core import configurations as cfg
from remotior_sensus.core.bandset_catalog import BandSet
from remotior_sensus.core.bandset_catalog import BandSetCatalog
from remotior_sensus.core.output_manager import OutputManager
from remotior_sensus.core.processor_functions import raster_erosion
from remotior_sensus.util import shared_tools


def band_erosion(
        input_bands: Union[list, int, BandSet], value_list: list, size: int,
        output_path: Union[list, str] = None,
        overwrite: Optional[bool] = False,
        circular_structure: Optional[bool] = None,
        prefix: Optional[str] = '', extent_list: Optional[list] = None,
        multiple_resolution: Optional[bool] = True,
        n_processes: Optional[int] = None,
        available_ram: Optional[int] = None,
        bandset_catalog: Optional[BandSetCatalog] = None,
        virtual_output: Optional[bool] = None,
        progress_message: Optional[bool] = True
) -> OutputManager:
    """Perform erosion of band pixels.

    This tool performs the erosion of pixels identified by a list of values.
    A new raster is created for each input band.

    Args:
        input_bands: input of type BandSet or list of paths or integer
            number of BandSet.
        output_path: string of output path directory or list of paths.
        overwrite: if True, output overwrites existing files.
        value_list: list of values for dilation.
        size: size of dilation in pixels.
        virtual_output: if True (and output_path is directory), save output
            as virtual raster of multiprocess parts.
        circular_structure: if True, use circular structure; if False, square 
            structure.
        prefix: optional string for output name prefix.
        extent_list: list of boundary coordinates left top right bottom.
        multiple_resolution: if True, keep the original resolution of 
            individual raster; 
            if False, use the resolution of the first raster for all the bands.
        n_processes: number of parallel processes.
        available_ram: number of megabytes of RAM available to processes.
        bandset_catalog: optional type BandSetCatalog for BandSet number.
        progress_message: if True then start progress message, if False does 
            not start the progress message (useful if launched from other tools).

    Returns:
        Object :func:`~remotior_sensus.core.output_manager.OutputManager` with
            - paths = output list

    Examples:
        Perform the erosion of size 1 for value 1 and 2
            >>> erosion = band_erosion(
            ... input_bands=['path_1', 'path_2'], value_list=[1, 2], size=1,
            ... output_path='directory_path', circular_structure=True
            ... )
    """  # noqa: E501
    cfg.logger.log.info('start')
    cfg.progress.update(
        process=__name__.split('.')[-1].replace('_', ' '), message='starting',
        start=progress_message
    )
    # prepare process files
    prepared = shared_tools.prepare_process_files(
        input_bands=input_bands, output_path=output_path, overwrite=overwrite,
        n_processes=n_processes, bandset_catalog=bandset_catalog,
        prefix=prefix, box_coordinate_list=extent_list,
        multiple_output=True,  multiple_input=True,
        multiple_resolution=multiple_resolution,
        virtual_output=virtual_output
    )
    input_raster_list = prepared['input_raster_list']
    raster_info = prepared['raster_info']
    n_processes = prepared['n_processes']
    nodata_list = prepared['nodata_list']
    output_list = prepared['output_list']
    vrt_list = prepared['vrt_list']
    if not circular_structure:
        structure = shared_tools.create_base_structure(3)
    else:
        structure = shared_tools.create_circular_structure(1)
    # process calculation
    n = 0
    min_p = 1
    max_p = int((99 - 1) / len(input_raster_list))
    for i in input_raster_list:
        cfg.progress.update(message='processing raster %s' % (n + 1))
        out = output_list[n]
        nd = nodata_list[n]
        data_type = raster_info[n][8]
        # dummy bands for memory calculation
        dummy_bands = 7
        cfg.multiprocess.run(
            raster_path=i, function=raster_erosion,
            function_argument=structure, function_variable=[size, value_list],
            output_raster_path=out, output_data_type=data_type,
            n_processes=n_processes, available_ram=available_ram,
            output_nodata_value=nd, compress=cfg.raster_compression,
            dummy_bands=dummy_bands, boundary_size=structure.shape[0] + 1,
            virtual_raster=vrt_list[n],
            progress_message='processing raster %s' % (n + 1),
            min_progress=min_p + max_p * n,
            max_progress=min_p + max_p * (n + 1)
        )
        n += 1
    cfg.progress.update(end=True)
    cfg.logger.log.info('end; band erosion: %s' % output_list)
    return OutputManager(paths=output_list)



from pathlib import Path
from unittest import TestCase

import remotior_sensus


class TestRasterToVector(TestCase):

    def test_raster_to_vector(self):
        rs = remotior_sensus.Session(
            n_processes=2, available_ram=1000, log_level=10
            )
        cfg = rs.configurations
        data_path = Path(__file__).parent / 'data'
        p = str(data_path / 'S2_2020-01-01' / 'S2_B02.tif')
        temp = cfg.temp.temporary_file_path(name_suffix=cfg.gpkg_suffix)
        cfg.logger.log.debug('>>> test raster_to_vector')
        vector = rs.raster_to_vector(p, temp)
        self.assertTrue(rs.files_directories.is_file(vector.path))
        temp = cfg.temp.temporary_file_path(name_suffix=cfg.gpkg_suffix)
        cfg.logger.log.debug('>>> test raster_to_vector dissolve')
        vector = rs.raster_to_vector(p, temp, dissolve=True)
        self.assertTrue(rs.files_directories.is_file(vector.path))

        # clear temporary directory
        rs.close()
